import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.*;
import com.mindfusion.drawing.SolidBrush;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.text.html.HTMLEditorKit;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

public class MainWindow extends JFrame implements ShapeListBoxListener, ActionListener {

	
	public static void main(String[] args) {
        SwingUtilities.invokeLater(() -> {
            MainWindow window = new MainWindow();
            if (show)
                window.setVisible(true);
            else
                window.dispose();

        });
    }

    private MainWindow() {

        setLayout(new BorderLayout());
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setLocationByPlatform(true);
        setSize(900, 700);
        setTitle("JDiagram sample: MySQL Storage");

        Credentials credentials = Credentials.getInstance();
        if (credentials.exists()) {
            credentials.assignCredentials();
            show = true;
        } else {
            ConnectionWindow connectionWindow = new ConnectionWindow(this);
            connectionWindow.display();
            show = connectionWindow.canDisplay;
        }

        if (show)
            DBConnection.config();

        JTextPane infoPane = new JTextPane();
        infoPane.setBackground(new Color(254, 255, 211));
        infoPane.setBorder(new LineBorder(Color.orange, 1));
        infoPane.setEditorKit(new HTMLEditorKit());
        infoPane.setEditable(false);

        JScrollPane infoScrollPane = new JScrollPane(infoPane);
        infoScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        infoScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        diagram = new Diagram();
        diagram.setLinkBaseShapeSize((float) 5);
        diagram.setLinkHeadShapeSize((float) 5);
        diagram.addDiagramListener(new ConcreteEventListener());
        diagram.setBackBrush(new SolidBrush(Color.decode("#CAE0C9")));
        diagram.getShapeNodeStyle().setBrush(new SolidBrush(Color.decode("#34DE7E")));


        diagramView = new DiagramView();
        diagramView.setBehavior(Behavior.DoNothing);
        diagramView.setDiagram(diagram);
        diagramView.setAllowInplaceEdit(true);

        JScrollPane diagramPane = new JScrollPane(diagramView);

        JPanel rightPanel = new JPanel(new VerticalFlowLayout(
                VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 5, 0));

        shapeListBox = new ShapeListBox();
        shapeListBox.setEnabled(false);
        shapeListBox.setShapeFillColor(Color.decode("#34DE7E"));
        shapeListBox.setDragEnabled(false);
        shapeListBox.addShapeListener(this);

        JScrollPane shapeListPane = new JScrollPane(shapeListBox);
        shapeListPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        shapeListPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        linkShapeBox = new JComboBox<>(new String[]{"Bezier", "Polyline", "Cascading", "Spline"});
        linkShapeBox.setEnabled(false);
        linkShapeBox.setSelectedIndex(1);
        linkShapeBox.setActionCommand("shape_box");
        linkShapeBox.addActionListener(this);

        JPanel linkDataPanel = new JPanel();
        linkDataPanel.setLayout(new BorderLayout());

        linkDataPanel.add(new JLabel("Link Shapes: "), BorderLayout.WEST);
        linkDataPanel.add(linkShapeBox, BorderLayout.CENTER);

        rightPanel.add(infoScrollPane);
        rightPanel.add(Box.createVerticalStrut(5));
        rightPanel.add(shapeListPane);
        rightPanel.add(Box.createVerticalStrut(5));
        rightPanel.add(linkDataPanel, Integer.valueOf(VerticalFlowLayout.STRETCH));


        setInfo(infoPane);
        setMenuItems();


        getContentPane().add(diagramPane, BorderLayout.CENTER);
        getContentPane().add(rightPanel, BorderLayout.EAST);
        getContentPane().add(infoScrollPane, BorderLayout.NORTH);


    }

    private void setMenuItems() {

        JMenuBar bar = new JMenuBar();

        JMenu menu = new JMenu("File");

        JMenuItem newItem = new JMenuItem("New");
        newItem.addActionListener(this);
        newItem.setActionCommand("new");

        JMenuItem saveItem = new JMenuItem("Save");
        saveItem.addActionListener(this);
        saveItem.setActionCommand("save");

        JMenuItem loadItem = new JMenuItem("Load");
        loadItem.addActionListener(this);
        loadItem.setActionCommand("load");

        menu.add(newItem);
        menu.add(saveItem);
        menu.add(loadItem);

        bar.add(menu);

        setJMenuBar(bar);


    }

    private void setInfo(JTextPane pane) {

        pane.setText("<div style=\"padding: 2pt; font-family: Verdana; font-size: 11pt;\">"
                + "This sample demonstrates how diagrams could be stored in a "
                + "<strong>MySql</strong> database using <strong>JDBC</strong> connector. "
                + "Create a diagram using the File -> New command, then draw nodes "
                + "and links on the diagram canvas. The controls on the right let you "
                + "select default shapes for new diagram items."
                + "</div>");
    }

    @Override
    public void shapeSelected(Shape shape, int i) {
        diagram.setDefaultShape(shape);
    }

    @Override
    public void shapeRenamed(Shape shape, int i, String s) {

    }

    @Override
    public void shapeRemoved(Shape shape, int i) {

    }

    @Override
    public void actionPerformed(ActionEvent e) {

        switch (e.getActionCommand()) {
            case "shape_box":
                switch (linkShapeBox.getSelectedIndex()) {
                    case 0:
                        diagram.setLinkShape(LinkShape.Bezier);
                        break;
                    case 1:
                        diagram.setLinkShape(LinkShape.Polyline);
                        break;
                    case 2:
                        diagram.setLinkShape(LinkShape.Cascading);
                        break;
                    case 3:
                        diagram.setLinkShape(LinkShape.Spline);
                        break;
                }
                break;
            case "new":
                String name = JOptionPane.showInputDialog(
                        this,
                        "Enter Name",
                        "Enter Diagram name", JOptionPane.PLAIN_MESSAGE);
                if (name == null || name.isEmpty())
                    return;
                Crud.getDiagrams();
                if (Crud.diagramValueMap.containsKey(name)) {
                    JOptionPane.showMessageDialog(this,
                            "Diagram with same name already exists in database",
                            "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Crud.savedNodesMap.clear();
                Crud.updatedNodeMap.clear();
                Crud.deletedNodeMap.clear();
                Crud.savedLinksMap.clear();
                Crud.updatedLinkMap.clear();
                Crud.deletedLinkMap.clear();
                diagram = onCreateDiagram();

                diagram.setTag(name);
                diagramView.setBehavior(Behavior.LinkShapes);
                shapeListBox.setEnabled(true);
                linkShapeBox.setEnabled(true);
                diagramView.repaint();
                break;
            case "save":
                if (diagram.getNodes().size() > 0)
                    Crud.save(diagram);
                break;
            case "load":
                onLoadDiagrams();
                break;
        }
    }


    private Diagram onCreateDiagram() {

        diagram = new Diagram();
        diagram.setLinkBaseShapeSize((float) 5);
        diagram.setLinkHeadShapeSize((float) 5);
        diagram.addDiagramListener(new ConcreteEventListener());
        diagram.setBackBrush(new SolidBrush(Color.decode("#CAE0C9")));
        diagram.getShapeNodeStyle().setBrush(new SolidBrush(Color.decode("#34DE7E")));
        diagramView.setDiagram(diagram);

        return diagram;
    }

    private void onLoadDiagrams() {
        Vector<String> diagrams = Crud.getDiagrams();
        if (diagrams.isEmpty())
            return;
        JList<String> nData = new JList<>(diagrams);
        showSelectionList(nData);

    }

    private void showSelectionList(JList<String> sData) {

        JDialog dialog = new JDialog();
        dialog.setSize(200, 300);
        dialog.setResizable(false);
        dialog.setAlwaysOnTop(true);

        JScrollPane scrollPane = new JScrollPane(sData,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        dialog.getContentPane().add(scrollPane);
        dialog.setLocationRelativeTo(this);
        dialog.setVisible(true);
        sData.addListSelectionListener(e -> {
            if (e.getValueIsAdjusting()) {
                diagram = onCreateDiagram();
                if (Crud.load(sData.getSelectedIndex() + 1, sData.getSelectedValue(), diagram)) {
                    diagram.setTag(sData.getSelectedValue());
                    diagramView.setBehavior(Behavior.LinkShapes);
                    shapeListBox.setEnabled(true);
                    linkShapeBox.setEnabled(true);
                }
                diagram.repaint();
                diagramView.repaint();
                diagramView.revalidate();
                dialog.dispose();
            }

        });
    }

    class ConcreteEventListener extends DiagramAdapter {

        @Override
        public void nodeCreated(NodeEvent nodeEvent) {
            DiagramNode node = nodeEvent.getNode();
            String id = diagram.getDefaultShape().getDisplayName() + UUID.randomUUID().toString();
            node.setId(id);

        }

        @Override
        public void nodeDeleted(NodeEvent nodeEvent) {
            if (nodeEvent.getNode().getTag() != null) {
                Crud.deletedNodeMap.put((String) nodeEvent.getNode().getId(), nodeEvent.getNode());
                Crud.updatedNodeMap.remove((String) nodeEvent.getNode().getId());
            }
        }

        @Override
        public void nodeModified(NodeEvent nodeEvent) {
            if (nodeEvent.getNode().getTag() != null)
                Crud.updatedNodeMap.put((String) nodeEvent.getNode().getId(), nodeEvent.getNode());
        }

        @Override
        public void linkCreated(LinkEvent linkEvent) {
            DiagramLink link = linkEvent.getLink();
            String id = diagram.getLinkShape().name() + UUID.randomUUID().toString();
            link.setId(id);

        }

        @Override
        public void linkModified(LinkEvent linkEvent) {
            if (linkEvent.getLink().getTag() != null)
                Crud.updatedLinkMap.put((String) linkEvent.getLink().getId(), linkEvent.getLink());

        }

        @Override
        public void linkDeleted(LinkEvent linkEvent) {
            if (linkEvent.getLink().getTag() != null) {
                Crud.deletedLinkMap.put((String) linkEvent.getLink().getId(), linkEvent.getLink());
                Crud.updatedLinkMap.remove(linkEvent.getLink().getId().toString());
            }
        }

        @Override
        public void leaveInplaceEditMode(InPlaceEditEvent inPlaceEditEvent) {
            if (inPlaceEditEvent.getItem() instanceof DiagramLink) {
                DiagramLink link = (DiagramLink) inPlaceEditEvent.getItem();
                if (link.getTag() != null) {
                    Crud.updatedLinkMap.put((String) link.getId(), link);
                }
            } else {
                DiagramNode node = (DiagramNode) inPlaceEditEvent.getItem();
                if (node.getTag() != null)
                    Crud.updatedNodeMap.put((String) node.getId(), node);
            }
        }
    }

    private Diagram diagram;
    private DiagramView diagramView;
    private JComboBox<String> linkShapeBox;
    private ShapeListBox shapeListBox;
    private static boolean show;
    private static final long serialVersionUID = 4410103780809349264L;

}
