import java.io.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.geom.Point2D;

import javax.swing.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


class FlowChartTransferHandler
	extends TransferHandler
{
	private class FormTransferable implements Transferable
	{
		FormTransferable(Image image)
		{
			_image = image;
		}

		public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException
		{

			if (!isDataFlavorSupported(flavor))
				throw new UnsupportedFlavorException(flavor);

			return _image;
		}

		public DataFlavor[] getTransferDataFlavors()
		{

			return new DataFlavor[]
			{
				DataFlavor.imageFlavor,
				DataFlavor.stringFlavor
			};
		}

		public boolean isDataFlavorSupported(DataFlavor flavor)
		{
			return DataFlavor.imageFlavor.equals(flavor) ||
				DataFlavor.stringFlavor.equals(flavor);
		}

		private Image _image;
	}

	protected Transferable createTransferable(JComponent c)
	{
		if (c instanceof ImageBox)
			return new FormTransferable(((ImageBox)c).getImage());
		else
			return new StringSelection(((JList<?>)c).getSelectedValue().toString());
	}

	public boolean importData(JComponent c, Transferable t)
	{
		if (canImport(c, t.getTransferDataFlavors()))
		{
			// support only dropping over the diagram
			if (!(c instanceof DiagramView))
				return false;

			// get the target diagram
			DiagramView diagramView = (DiagramView)c;
			Diagram diagram = diagramView.getDiagram();

			// get mouse position in diagram coordinate system
			Point mouseLocation = MouseInfo.getPointerInfo().getLocation();
			SwingUtilities.convertPointFromScreen(
				mouseLocation, diagramView);
			Point2D nodePos = diagramView.deviceToDoc(mouseLocation);

			if (t.isDataFlavorSupported(DataFlavor.imageFlavor))
			{
				try
				{
					Image image = (Image)t.getTransferData(DataFlavor.imageFlavor);

					// create a node that displays the dropped image
					ShapeNode node = diagram.getFactory().createShapeNode(
						nodePos.getX() - 10, nodePos.getY() - 10, 20, 20);
					node.setImage(image);
					node.setImageAlign(ImageAlign.Fit);
					node.setTransparent(true);
					node.setRotateImage(true);
					node.setRotateText(true);

					return true;
				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
			else if (t.isDataFlavorSupported(DataFlavor.stringFlavor))
			{
				try
				{
					String string = (String)t.getTransferData(DataFlavor.stringFlavor);
					Shape shape = FlowCharterFrame.getNodeShape();

					ShapeNode node = diagram.getFactory().createShapeNode(
						nodePos.getX() - 10, nodePos.getY() - 10, 20, 20);

					AnchorPattern ap = FlowCharterFrame.getAnchorPattern();

					for(AnchorPoint point : ap.getPoints()){
						point.setMarkStyle(FlowCharterFrame.getAnchorStyle());
						point.setColor(FlowCharterFrame._anchorColor);
					}

					if (shape != null)
						node.setShape(shape);
					else
						node.setText(string);

					node.setAnchorPattern(ap);
					node.setHandlesStyle(HandlesStyle.DashFrame);
				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
		}

		return false;
	}

	public int getSourceActions(JComponent c)
	{
		return COPY;
	}

	protected void exportDone(JComponent c, Transferable data, int action)
	{
	}

	public boolean canImport(JComponent c, DataFlavor[] flavors)
	{
		if (!(c instanceof DiagramView))
			return false;

		for (int i = 0; i < flavors.length; i++)
		{
			if (DataFlavor.imageFlavor.equals(flavors[i]))
				return true;

			if (DataFlavor.stringFlavor.equals(flavors[i]))
				return true;
		}

		return false;
	}

	private static final long serialVersionUID = 1L;
}
