/**
 * Copyright (c) 2014-2024, MindFusion LLC - Bulgaria.
 */

import java.awt.event.*;
import java.awt.geom.*;
import java.io.*;

import javax.swing.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.jlayout.TreeLayoutDirection;


public class Tutorial2 extends JFrame
{
	public static void main(String[] args)
	{
		Tutorial2 mainFrame = new Tutorial2();
		mainFrame.setVisible(true);
	}

	public Tutorial2()
	{
		super("JDiagram Tutorial 2: Tree from XML");

		// set up the main window
		setBounds(0, 0, 800, 550);
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		// create diagram and view
		final Diagram diagram = new Diagram();
		DiagramView view = new DiagramView();
		view.setDiagram(diagram);

		// add scroll pane
		JScrollPane scrollPane = new JScrollPane(view);
		scrollPane.setVisible(true);
		getContentPane().add(scrollPane);
		
		this.addWindowListener(new WindowAdapter()
		{
			@Override
			public void windowOpened(WindowEvent e)
			{
				loadTree(diagram, "SampleTree.xml");
			}
		});
	}
	
	void loadTree(Diagram diagram, String filepath)
	{
		// load the tree XML
		Document document = loadXmlFile(filepath);
		Element rootElement = document.getDocumentElement();

		// create diagram elements recursively
		ShapeNode root = diagram.getFactory().createShapeNode(bounds);
		root.setText("Project");
		createChildren(diagram, root, rootElement);

		// arrange using TreeLayout
		TreeLayout layout = new TreeLayout();
		layout.setType(TreeLayoutType.Cascading);
		layout.setDirection(TreeLayoutDirection.LeftToRight);
		layout.setLinkStyle(TreeLayoutLinkType.Cascading2);
		layout.setNodeDistance(3);
		layout.setLevelDistance(-8); // let horizontal positions overlap
		layout.arrange(diagram);
	}
	
	void createChildren(Diagram diagram, DiagramNode parentNode, Node parentElement)
	{
		NodeList elements = parentElement.getChildNodes();
		for (int i = 0; i < elements.getLength(); i++)
		{
			Node element = elements.item(i);
			if (element.getNodeName().equals("Activity"))
			{
				ShapeNode node = diagram.getFactory().createShapeNode(bounds);
				node.setText(((Element)element).getAttribute("Name"));
				diagram.getFactory().createDiagramLink(parentNode, node);
				createChildren(diagram, node, element);
			}
		}
	}

	Document loadXmlFile(String filepath)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringElementContentWhitespace(true);
		factory.setNamespaceAware(true);

		Document document = null;
		DocumentBuilder builder;
		try
		{
			File file = new File(filepath);
			builder = factory.newDocumentBuilder();
			document = builder.parse(file); 
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return document;
	}
	
	final Rectangle2D.Float bounds = new Rectangle2D.Float(0,0,30,8);
	private static final long serialVersionUID = 1L;
}