/**
 * Copyright (c) 2014-2024, MindFusion LLC - Bulgaria.
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.*;

import javax.imageio.ImageIO;
import javax.swing.*;

import com.mindfusion.diagramming.*;


public class Tutorial4 extends JFrame
{
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
	        public void run() {
	          new Tutorial4();
	        }
	    });
	}

	public Tutorial4()
	{
		super("JDiagram Tutorial 4: Extended Custom Nodes");

		// set up the main window
		setBounds(0, 0, 1000, 550);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setVisible(true);

		// set up main menu
		JMenuBar mBar = new JMenuBar();
		JMenu mbtnFile = new JMenu();
		JMenu mbtnEdit = new JMenu();
		JMenuItem miLoadBinary = new JMenuItem("Load Binary");
		JMenuItem miSaveBinary = new JMenuItem("Save Binary");
		JMenuItem miUndo = new JMenuItem("Undo");
		JMenuItem miRedo = new JMenuItem("Redo");
		
		mbtnFile.add(miLoadBinary);
		mbtnFile.add(miSaveBinary);
		mbtnFile.setText("File");
		mBar.add(mbtnFile);
		
		mbtnEdit.add(miUndo);
		mbtnEdit.add(miRedo);
		mbtnEdit.setText("Edit");
		mBar.add(mbtnEdit);
		
		this.setJMenuBar(mBar);

		// create diagram and view
		Diagram.registerClass(OrgChartNode.class, "my:OrgChartNode", 1);
		final Diagram diagram = new Diagram();
		diagram.getUndoManager().setUndoEnabled(true);
		DiagramView view = new DiagramView();
		view.setDiagram(diagram);

		// enable drawing OrgChartNodes with the mouse
		view.setCustomBehavior(new OrgChartBehavior(view));

		// add scroll pane
		JScrollPane scrollPane = new JScrollPane(view);
		scrollPane.setVisible(true);
		scrollPane.setSize(1000, 550);
		getContentPane().add(scrollPane);
		
		// create a NodeListView
		NodeListView nodeList = new NodeListView();
		nodeList.setSize(110, 800);
		nodeList.setDefaultNodeSize(new Dimension(80,40));
		nodeList.setIconSize(new Dimension(60, 30));
		nodeList.setIconMargin(new Dimension(5, 5));
		getContentPane().add(nodeList, BorderLayout.WEST);
		
		// add nodes to list view
		OrgChartNode orgNode = new OrgChartNode(diagram);
		orgNode.setTitle("CEO");
		orgNode.setFullName("Type here");
		orgNode.setText("Chief executive officer");
		orgNode.setImage(loadImage("image1.png"));
		nodeList.addNode(orgNode);
		
		orgNode = new OrgChartNode(diagram);
		orgNode.setTitle("CIO");
		orgNode.setFullName("Type here");
		orgNode.setText("Chief information officer");
		orgNode.setImage(loadImage("image2.png"));
		nodeList.addNode(orgNode);
		
		diagram.addDiagramListener(new DiagramAdapter() {
			@Override
			public void nodeCreated(NodeEvent e)
			{
				OrgChartNode node = (OrgChartNode) e.getNode();
				if (node != null && node.getImage() == null)
				{
					node.setImage(loadImage("image1.png"));
					node.setTitle("new node");
					node.setFullName("new node");
					node.setText("new node");
				}
			}
		});

		miLoadBinary.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				JFileChooser fileChooser = new JFileChooser();
				int returnValue = fileChooser.showOpenDialog(Tutorial4.this);
				if(returnValue == JFileChooser.APPROVE_OPTION)
				{
					File file = fileChooser.getSelectedFile();
					try
					{
						diagram.loadFrom(file.getAbsolutePath());
					} catch (IOException e1)
					{
						e1.printStackTrace();
					}
				}
			}			
		});

		miSaveBinary.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent e)
			{
				JFileChooser fileChooser = new JFileChooser();
				int returnValue = fileChooser.showSaveDialog(Tutorial4.this);
				if(returnValue == JFileChooser.APPROVE_OPTION)
				{
					File file = fileChooser.getSelectedFile();
					try
					{
						diagram.saveTo(file.getAbsolutePath());
					} catch (IOException e1)
					{
						e1.printStackTrace();
					} 
				}
			}
		});
		
		miUndo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				diagram.getUndoManager().undo();
			}
		});
		
		miRedo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				diagram.getUndoManager().redo();
			}
		});
	}
	
	BufferedImage loadImage(String fileName)
	{
		BufferedImage image = null;
		try
		{
			image = ImageIO.read(new File(fileName));
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
		return image;
	}
	
	private static final long serialVersionUID = 1L;
}